<?php

declare(strict_types = 1);

/**
 * Copyright (C) 2024 PRONOVIX GROUP.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 */

namespace Drupal\symfony_webhook_receiver\DependencyInjection;

use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
use Symfony\Component\DependencyInjection\Compiler\ServiceLocatorTagPass;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\Reference;
use Symfony\Component\RemoteEvent\Messenger\ConsumeRemoteEventHandler;

/**
 * Compiler pass responsible for registering remote event consumers.
 *
 * @internal This class is not part of the module's public programming API.
 */
final class RegisterRemoveEventConsumersCompilerPass implements CompilerPassInterface {

  /**
   * {@inheritdoc}
   */
  public function process(ContainerBuilder $container): void {
    $consumer_remote_event_handler = $container->getDefinition(ConsumeRemoteEventHandler::class);

    $services = [];
    foreach ($container->findTaggedServiceIds('remote_event.consumer') as $id => $tags) {
      $consumer_type = $tags[0]['type'];
      assert(!empty($consumer_type), sprintf('The "type" attribute is set on the "%s" remote event consumer.', $id));
      $services[$consumer_type] = new Reference($id);
    }

    $consumer_remote_event_handler->addArgument(ServiceLocatorTagPass::register($container, $services));
  }

}
